#include-once
; AutoIt Version: 3.0
; Language:       English
; Platform:       Win9x/NT
; Author:         Michael W. Bayley
;
; Script Function:
;   MultiRunFunc - MultiRun Support Functions
;

; Includes
#include "MultiRunDefs.au3"

;
; User Defined Functions
; Support Functions
;

;
; LaunchProgram($program, $directory, $params. $single, $mode)
; Entry:		$program	Program to run
;			    $directory	Start directory
;			    $params		Command line parameters
;			    $single		Limit to single instance
;			    $mode		Window mode
; Return:		Process ID of program (-1 if error)
;			    @error = -1 if program was already running (single instance)
;			    @error = 1 if program does not exist
;			    @error = 2 if directory does not exist
;			    @error = 3 if directory is not a directory
;			    @error = 4 if program failed to run
; Procedure:	1. Error if program does not exists.
;			    2. Error if directory does not exist.
;			    3. Error if directory is not a directory.
;			    4. Run the program.
;			    5.Return results.
;
Func LaunchProgram($program, $directory, $params, $single, $mode)
	Local	$pid, $attr, $file

	; 1. Error if program does not exists.
	$pid = -1
	If FileExists($program) == 0 Then
		SetError(1)


	; 2. Error if directory does not exist.
	ElseIf FileExists($directory) == 0 Then
		SetError(2)


	; 3. Error if directory is not a directory.
	ElseIf StringInStr(FileGetAttrib($directory), "D") == 0 Then
		SetError(3)


	; 4. Run the program.
	Else
		If $single Then
			If StringInStr($program, "\") == 0 Then
				$file = $program
			Else
				$file = StringMid($program, StringInStr($program, "\", 0, -1) + 1)
			EndIf
			
			$pid = ProcessExists($file)
			If $pid Then
				SetError(-1)
				Return($pid)
			EndIf
		EndIf

		If StringLen($directory) AND StringLen($params) Then
			If $mode == "" Then
				$pid = Run(Chr(34) & $program & Chr(34) & " " & $params, $directory)
			Else
				$pid = Run(Chr(34) & $program & Chr(34) & " " & $params, $directory, $mode)
			EndIf
		ElseIf StringLen($directory) Then
			If $mode == "" Then
				$pid = Run(Chr(34) & $program & Chr(34), $directory)
			Else
				$pid = Run(Chr(34) & $program & Chr(34), $directory, $mode)
			EndIf
		ElseIf StringLen($params) Then
			If $mode == "" Then
				$pid = Run(Chr(34) & $program & Chr(34) & " " & $params)
			Else
				$pid = Run(Chr(34) & $program & Chr(34) & " " & $params, "", $mode)
			EndIf
		Else
			If $mode == "" Then
				$pid = Run(Chr(34) & $program & Chr(34))
			Else
				$pid = Run(Chr(34) & $program & Chr(34), "", $mode)
			EndIf
		EndIf
		If @error Then SetError(4)
	EndIf

	; 5.Return results.
	Return($pid)
EndFunc


;
; WaitProgramWindow($program, $titleMode, $title1, $title2)
; Entry:		$program	Program to wait for
;			    $titleMode	Title match mode
;			    $title1		Title start text
;			    $title2		Title end text
; Return:		Title search string (NULL if error or not match mode disabled)
;			    @error = 1 if program not detedted
;			    @error = 2 if invalid search string
;			    @error = 3 if window not detected
; Procedure:	1. Wait for program process, error if fail.
;			    2. Exit if title disabled.
;			    3. Build title search string.
;			    4. Wait for title, error if fail.
;			    5. Return all ok.
;
Func WaitProgramWindow($program, $titleMode, $title1, $title2)
	Local	$file, $searchText

	; 1. Wait for program process, error if fail.
	If StringInStr($program, "\") == 0 Then
		$file = $program
	Else
		$file = StringMid($program, StringInStr($program, "\", 0, -1) + 1)
	EndIf

	If ProcessWait($file, 3) == 0 Then
		SetError(1)
		Return("")
	EndIf


	; 2. Exit if title disabled.
	If $titleMode == $IniRunTitleModeNone Then
		Return("")
	EndIf


	; 3. Build title search string.
	If $titleMode == $IniRunTitleModeRegexp Then
		$searchText = $RegexpHeader
		If StringLen($title1) AND StringLen($title2) Then
			$searchText &= $RegexpLeft1
			$searchText &= RegexpFix($title1)
			$searchText &= $RegexpLeft2
			$searchText &= $RegexpMid
			$searchText &= $RegexpRight1
			$searchText &= RegexpFix($title2)
			$searchText &= $regexpRight2
		ElseIf StringLen($title1) Then
			$searchText &= $RegexpLeft1
			$searchText &= RegexpFix($title1)
			$searchText &= $RegexpLeft2
			$searchText &= $RegexpMid
		ElseIf StringLen($title2) Then
			$searchText &= $RegexpMid
			$searchText &= $RegexpRight1
			$searchText &= RegexpFix($title2)
			$searchText &= $regexpRight2
		Else
			SetError(2)
			Return("")
		EndIf
	Else
		$searchText = $Title1
		If StringLen($searchText) == 0 Then
			SetError(2)
			Return("")
		EndIf
	EndIf


	; 4. Wait for title, error if fail.
	If WinWait($searchText, "", 3) == 0 Then
		SetError(3)
		Return("")
	EndIf


	; 5.Return all ok.
	Return($searchText)
EndFunc


;
; PositionProgramWindow($title, $width, $height, $xpos, $ypos)
; Entry:		$title		Title search string
;			    $width		Window width
;			    $height		Window height
;			    $xpos		Window X position
;			    $ypos		Window Y position
; Return:		1 if positioned ok
;			    0 if failed
;			    @error = 1 if window not found
; Procedure:	1. Exit if not positioning.
;			    2. Compute window size & position
;			    3. Position the window.
;			    4. Return all ok.
;
Func PositionProgramWindow($title, $width, $height, $xpos, $ypos)
	Local	$win[4]
	Local	$x, $y, $w, $h

	; 1. Exit if not positioning.
	If StringLen($title) == 0 Then Return(1)
	If $width == $IniRunDefault AND $height == $IniRunDefault AND $xpos == $IniRunDefault AND $ypos == $IniRunDefault Then Return(1)


	; 2. Compute window size & position
	$win = WinGetPos($title)
	If @error Then
		SetError(1)
		Return(0)
	EndIf
	
	If $xpos == $IniRunDefault Then
		If $width == $IniRunDefault Then
			$x = $win[0]
			$w = $win[2]
		ElseIf $width == $IniRunCenter Then
			If $win[0] < (@DesktopWidth / 2) Then
				$x = $win[0]
				$w = ((@DesktopWidth / 2) - $x) * 2
			Else
				$x = $win[0]
				$w = $win[2]
			EndIf
		Else
			$x = $win[0]
			$w = $width
		EndIf
	ElseIf $xpos == $IniRunCenter Then
		If $width == $IniRunDefault Then
			$w = $win[2]
			$x = (@DesktopWidth / 2) - ($w / 2)
		ElseIf $width == $IniRunCenter Then
			$w = $win[2]
			$x = (@DesktopWidth / 2) - ($w / 2)
		Else
			$w = $width
			$x = (@DesktopWidth / 2) - ($w / 2)
		EndIf
	Else
		If $width == $IniRunDefault Then
			$w = $win[2]
			$x = $xpos
		ElseIf $width == $IniRunCenter Then
			$x = $xpos
			If $x < (@DesktopWidth / 2) Then
				$w = ((@DesktopWidth / 2) - $x) * 2
			Else
				$w = $win[2]
			EndIf
		Else
			$w = $width
			$x = (@DesktopWidth / 2) - ($w / 2)
		EndIf
	EndIf

	If $ypos == $IniRunDefault Then
		If $height == $IniRunDefault Then
			$y = $win[1]
			$h = $win[3]
		ElseIf $height == $IniRunCenter Then
			If $win[0] < (@DesktopHeight / 2) Then
				$y = $win[1]
				$h = ((@DesktopHeight / 2) - $x) * 2
			Else
				$y = $win[1]
				$h = $win[3]
			EndIf
		Else
			$y = $win[1]
			$h = $height
		EndIf
	ElseIf $ypos == $IniRunCenter Then
		If $height == $IniRunDefault Then
			$h = $win[3]
			$y = (@DesktopHeight / 2) - ($w / 2)
		ElseIf $height == $IniRunCenter Then
			$h = $win[3]
			$w = (@DesktopHeight / 2) - ($w / 2)
		Else
			$h = $height
			$y = (@DesktopHeight / 2) - ($w / 2)
		EndIf
	Else
		If $height == $IniRunDefault Then
			$h = $win[3]
			$y = $ypos
		ElseIf $height == $IniRunCenter Then
			$y = $ypos
			If $y < (@DesktopHeight / 2) Then
				$h = ((@DesktopHeight / 2) - $x) * 2
			Else
				$h = $win[3]
			EndIf
		Else
			$h = $height
			$y = (@DesktopHeight / 2) - ($w / 2)
		EndIf
	EndIf


	; 3. Position the window.
	If WinMove($title, "", $x, $y, $w, $h) <> 1 Then
		SetError(1)
		Return(0)
	EndIf


	; 4.Return all ok.
	Return(1)
EndFunc


;
; SendProgramKeys($title, $keys)
; Entry:		$title		Title search string
;			    $keys		Keystrokes to send to window
; Return:		1 if text sent ok
;			    0 if failed
;			    @error = 1 if window not found
; Procedure:	1. Exit if not sending text.
;			    2. Activate the window.
;			    3. Send the keys.
;			    4. Return all ok.
;
Func SendProgramKeys($title, $keys)
	; 1. Exit if not sending text.
	If StringLen($title) == 0 Then Return(1)
	If StringLen($keys) == 0 Then Return(1)


	; 2. Activate the window.
	WinActivate($title)


	; 3. Send the keys.
	Send($keys)


	; 4.Return all ok.
EndFunc


;
; WaitProgram($program, $waittime, $timeout)
; Entry:		$pid			Program to wait on
;			    $waittime		Time to wait.
;			    $timeout		Timeout for program to terminate
; Return:		1 if ok
;			    0 if failed
;			    @error = 1 if timeout
; Procedure:	1. Exit if not waiting.
;			    2. Wait time if time in seconds, and exit.
;			    3. Wait for program to close.
;			    4. Return results.
;
Func WaitProgram($program, $waittime, $timeout)
	Local	$file

	; 1. Exit if not waiting.
	If $waittime == 0 Then Return(1)


	; 2. Wait time if time in seconds, and exit.
	If $waittime <> $IniRunWaitClose Then
		Sleep($waittime * 1000)
	EndIf


	; 3. Wait for program to close.
	If StringInStr($program, "\") == 0 Then
		$file = $program
	Else
		$file = StringMid($program, StringInStr($program, "\", 0, -1) + 1)
	EndIf
	If ProcessWaitClose($file, $timeout) == 0 Then
		SetError(1)
		Return(0)
	EndIf


	; 4.Return results.
	Return(1)
EndFunc


;
; User Defined Functions
; Miscelaneous Functions
;

;
; RegexpFix($text)
; Entry:		$text		Text to parse
; Return:		Text adjusted for regular expression search
; Procedure:	1. Parse text.
;			    2. Return results.
;
Func RegexpFix($text)
	Local	$text2, $index

	; 1. Parse the text.
	$text2 = ""
	$index = 1
	While $index <= STringLen($text)
		If StringInStr($RegexpChars, StringMid($text, $index, 1)) Then
			$text2 &= $RegexpEscape
		EndIf
		$text2 &= StringMid($text, $index, 1)
		$index += 1
	WEnd


	; 2.Return results.
	Return($text2)
EndFunc

